"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.setCommonContext = exports.getCommonContext = void 0;
/**
 * 公共 context 信息, 与特定数据源无关的 context
 */
const tcb_1 = require("../tcb");
const config_1 = require("../config");
let cachedCommonContext;
function getCommonContext() {
    if (!cachedCommonContext) {
        console.warn('common context not initialed yet');
    }
    return cachedCommonContext;
}
exports.getCommonContext = getCommonContext;
/**
 * 设置公共的context
 */
function setCommonContext(config) {
    if (cachedCommonContext) {
        // 在云函数中存在对象共享的问题, 多次云函数调用可能用共享同一个内存对象
        // console.warn('<cachedCommonContext> should be called only once!');
        // return cachedCommonContext;
    }
    const parsedContext = tcb_1.tcb.parseContext(config.rawContext);
    const envInfo = getEnvInfo(config);
    cachedCommonContext = {
        callDataSource: config.callDataSource,
        cloudbase: tcb_1.tcb,
        app: tcb_1.app,
        auth: tcb_1.auth,
        env: envInfo,
        envInfo,
        database: tcb_1.db,
        weixin: getWxCloudInfo(config.cfnParams),
        wedaTarget: config.cfnParams.wedaTarget,
    };
    // 仅当预览云函数才能设置为测试模式
    const isTestMode = envInfo.isPreview && config.cfnParams.isTestMode;
    config_1.setConfig({
        getDataSources: config.getDataSources,
        wedaTarget: cachedCommonContext.wedaTarget,
        requestId: parsedContext.requestId,
        entrance: config.entrance,
        isTestMode,
        testDataSource: isTestMode && config.cfnParams.testDataSource,
        envType: envInfo.envType,
        userSource: getUserSource(envInfo, config),
    });
    return cachedCommonContext;
}
exports.setCommonContext = setCommonContext;
/**
 * 获取环境变量信息
 */
function getEnvInfo(config) {
    const envTypeInfo = getEnvType(config);
    const envObj = config_1.getCloudbaseContext(config.rawContext);
    return Object.assign(Object.assign({}, envTypeInfo), { tcbSource: envObj.TCB_SOURCE || '', wedaAppId: config.cfnParams.wedaAppId, 
        // @ts-ignore
        currentOpenId: envObj.WX_FROM_OPENID || envObj.WX_OPENID, 
        // @ts-ignore
        currentAppId: envObj.WX_FROM_APPID || envObj.WX_APPID, openId: envObj.WX_OPENID, 
        // @ts-ignore
        fromOpenId: envObj.WX_FROM_OPENID, appId: envObj.WX_APPID, 
        // @ts-ignore
        fromAppId: envObj.WX_FROM_APPID, uid: envObj.TCB_UUID, customUserId: envObj.TCB_CUSTOM_USER_ID || '', isAnonymous: envObj.TCB_ISANONYMOUS_USER === 'true', envId: envObj.TCB_ENV || envObj.SCF_NAMESPACE });
}
/**
 * 获取环境类型信息, 返回 { isPreview, envType }
 *  优先使用 config.isPreview 判断环境类型, 再使用环境变量, 最后才使用 params 中的 envType
 */
function getEnvType(config) {
    let isPreview;
    let { envType } = config.cfnParams;
    if (isPreview === undefined && process.env.IS_PREVIEW) {
        isPreview = true;
    }
    if (isPreview === undefined) {
        if (envType) {
            isPreview = envType === 'pre';
            // console.log('isPreview', isPreview, envType)
            return {
                isPreview,
                envType,
            };
        }
        isPreview = true;
    }
    envType = isPreview ? 'pre' : 'prod';
    return {
        isPreview,
        envType,
    };
}
/**
 * 获取微信侧用户授权信息
 * @param params 云函数额外参数
 * @returns
 */
function getWxCloudInfo(params) {
    const wxReg = /^wx_/;
    const wxKeys = Object.keys(params).filter(k => wxReg.test(k));
    if (!wxKeys.length)
        return null;
    const weixin = {};
    return wxKeys.reduce((acc, cur) => {
        acc[cur.replace(wxReg, '')] = params[cur];
        return acc;
    }, weixin);
}
/** 获取用户来源信息 */
function getUserSource(envInfo, config) {
    const openId = envInfo.currentOpenId;
    const tcbUid = envInfo.uid;
    if (!openId && !tcbUid) {
        // 测试模式固定用 wedaId 0
        if (config.cfnParams.isTestMode) {
            return {
                uid: '0',
                source: 4,
            };
        }
        /** 其他情况调用, 无法获胜用户信息 */
        // 来自本地调用或者被其他云函数调用, 此时允许使用传递过来的 source 信息
        // `,`结尾表示没有来源, 一般为本地调用
        if (/(^,|\bscf$)/i.test(envInfo.tcbSource)) {
            return config.cfnParams.userSource;
        }
        return;
    }
    return {
        uid: (openId || tcbUid),
        /** uid 类型, 1.tcb 2.微信 3.企业微信 4.weda内部 */
        source: openId ? 2 : 1,
    };
}
