"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.sendRequest = exports.isSuccessStatusCode = void 0;
const request_1 = __importDefault(require("request"));
const util_1 = __importDefault(require("util"));
const utils_1 = require("../utils");
const json_transform_1 = require("./json-transform");
const normalize_http_method_1 = require("./normalize-http-method");
function mergeFields(fields, fields1) {
    // one of them is empty
    if (!fields !== !fields1)
        return fields || fields1;
    // both are empty, return empty
    if (!fields)
        return fields;
    return Object.assign({}, fields, fields1);
}
/**
 * 解析URL地址
 *  这里不使用node的 url.resolve, 因为会与用户预期不一致 url.resolve('http://example.com/one', '/two');    // 'http://example.com/two'
 * 本算法采取字符串拼接方式来解析URL, 并能移除多余的 / , 也能自动补充缺少的 /
 */
function resolveUrl(baseUrl, url) {
    if (!baseUrl || /^https?:\/\//i.test(url || ''))
        return url;
    if (!url)
        return baseUrl;
    // 处理路径 / 问题
    const baseUrlEndsWithSlash = baseUrl.lastIndexOf('/') === baseUrl.length - 1;
    const urlStartsWithSlash = url.indexOf('/') === 0;
    if (baseUrlEndsWithSlash && urlStartsWithSlash) {
        return baseUrl + url.slice(1);
    }
    if (baseUrlEndsWithSlash || urlStartsWithSlash) {
        return baseUrl + url;
    }
    return `${baseUrl}/${url}`;
}
function isSuccessStatusCode(code) {
    return (code >= 200 && code < 300) || code === 304;
}
exports.isSuccessStatusCode = isSuccessStatusCode;
function hasHeader(headers, headerName) {
    const h = headerName.toLowerCase();
    return Object.keys(headers).some(k => k.toLowerCase() === h);
}
async function sendRequest(ds, rawMethodConfig, context, params) {
    var _a, _b, _c, _d;
    const dsConfig = ds.schema || {};
    const methodConfig = normalize_http_method_1.formatHttpMethodConfig(rawMethodConfig);
    if (methodConfig.type !== 'http')
        throw new Error(`method ${methodConfig.name} not a valid http datasource method`);
    const tplData = { params, env: context.env, vars: context.vars };
    const httpConfig = methodConfig.calleeBody;
    // transform param placeholders in url
    let requestUrl;
    try {
        requestUrl = json_transform_1.transformJSONWithTemplate(tplData, resolveUrl(dsConfig.baseUrl, httpConfig.url));
    }
    catch (error) {
        throw new Error('expression in request url is invalid');
    }
    if (!/^https?:\/\//i.test(requestUrl))
        throw new Error('invalid request url');
    const requestMethod = httpConfig.method.toUpperCase();
    const requestConfig = {
        // 避免URL中存在未编码的特殊字符, 导致 node 层直接报错 Request path contains unescaped characters, 拒绝发送请求
        uri: encodeURI(requestUrl),
        method: requestMethod,
        headers: {},
    };
    // TODO: 公共 header 处理, 产品实际并未支持该功能, 公共 header 字段名称可能需要调整
    if (dsConfig.header || ((_a = httpConfig.header) === null || _a === void 0 ? void 0 : _a.values)) {
        const headerFields = mergeFields(dsConfig.header, httpConfig.header.values);
        try {
            requestConfig.headers = utils_1.lowercaseWholeKey(json_transform_1.transformJSONWithTemplate(tplData, headerFields) || {});
        }
        catch (error) {
            throw new Error('expression in httpConfig.headers invalid');
        }
    }
    if (!hasHeader(requestConfig.headers, 'User-Agent')) {
        requestConfig.headers['User-Agent'] = 'CloudBaseLowCode/2.0';
    }
    // suggest server respond with a json string
    if (!hasHeader(requestConfig.headers, 'Accept')) {
        requestConfig.headers.Accept = 'application/json, text/javascript, */*; q=0.01';
    }
    if (['POST', 'PUT', 'PATCH'].includes(requestMethod) && ((_b = httpConfig.body) === null || _b === void 0 ? void 0 : _b.values)) {
        let body;
        let contentType;
        if ((_c = httpConfig.body) === null || _c === void 0 ? void 0 : _c.values) {
            try {
                body = json_transform_1.transformJSONWithTemplate(tplData, httpConfig.body.values);
                contentType = httpConfig.body.contentType;
            }
            catch (error) {
                throw new Error('expression in body or defaultParams.body invalid');
            }
        }
        else {
            body = {};
            contentType = 'json';
        }
        switch (contentType) {
            case 'form':
            case 'json':
                try {
                    // body 应为对象
                    const evalBody = typeof body === 'string' ? json_transform_1.evaluateCode(body, null) : body;
                    if (contentType === 'form') {
                        requestConfig.form = evalBody;
                    }
                    else {
                        requestConfig.body = evalBody;
                        requestConfig.json = true;
                    }
                }
                catch (error) {
                    throw new Error(`request body is invalid json string, request not sent: ${body}`);
                }
                break;
            case 'xml':
                // can not be merged
                requestConfig.body = String(body);
                requestConfig.headers['Content-Type'] = 'application/xml;charset=UTF-8';
                break;
            case 'raw':
            default:
                requestConfig.body = String(body);
        }
    }
    if ((_d = httpConfig.query) === null || _d === void 0 ? void 0 : _d.values) {
        try {
            const query = json_transform_1.transformJSONWithTemplate(tplData, httpConfig.query.values);
            requestConfig.qs = query;
        }
        catch (error) {
            throw new Error('expression in httpConfig.query invalid');
        }
    }
    // 强制将返回 body 类型改为 buffer
    requestConfig.encoding = null;
    let finalRequestConfig = requestConfig;
    if (context.httpAuth) {
        const authData = await context.httpAuth.getAuthData(requestConfig);
        finalRequestConfig = mergeAuthData(requestConfig, authData);
    }
    console.log('finalRequestConfig', finalRequestConfig);
    return new Promise((resolve, reject) => {
        request_1.default(finalRequestConfig, (error, response, body) => {
            if (error)
                return reject(error);
            // 处理 body 编码问题
            const contentType = response.caseless.get('content-type');
            let decodedBody = body;
            if (Buffer.isBuffer(body)) {
                let charset = 'utf8';
                if (contentType && /\bcharset\s*=\s*([^\s;]+)/i.test(contentType)) {
                    charset = RegExp.$1.toLowerCase();
                }
                try {
                    const decoder = new util_1.default.TextDecoder(charset, { fatal: true });
                    decodedBody = decoder.decode(body);
                }
                catch (error) {
                    console.warn(`unable to decode or not text based content: ${contentType}`);
                }
            }
            if (error) {
                return reject(error);
            }
            if (!isSuccessStatusCode(response.statusCode)) {
                const errorBody = typeof decodedBody !== 'string' ? JSON.stringify(decodedBody) : decodedBody;
                return reject(new Error(`http statusCode ${response.statusCode}, response body ${errorBody}`));
            }
            response.body = decodedBody;
            resolve({ response, body: decodedBody });
        });
    });
}
exports.sendRequest = sendRequest;
function mergeAuthData(requestConfig, authData) {
    const newConfig = Object.assign({}, requestConfig);
    if (authData.uri) {
        newConfig.uri = authData.uri;
    }
    if (authData.body) {
        if (requestConfig.form) {
            newConfig.form = Object.assign(newConfig.form || {}, authData.body);
        }
        else if (!newConfig.body || typeof newConfig.body === 'object') {
            newConfig.body = Object.assign(newConfig.body || {}, authData.body);
            // 再次设置 contentType 为json, 避免 requestConfig.body 为空时未设置 contentType 的问题
            newConfig.json = true;
        }
    }
    if (authData.headers) {
        newConfig.headers = Object.assign(newConfig.headers || {}, authData.headers);
    }
    if (authData.qs) {
        newConfig.qs = Object.assign(newConfig.qs || {}, authData.qs);
    }
    return newConfig;
}
