"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.hasMethodPermission = exports.getMethodCfg = exports.invokeDataSource = void 0;
const fn_context_1 = require("./fn-context");
const builtin_method_1 = require("./builtin-method");
const http_method_1 = require("./http-method");
const cloudfn_method_1 = require("./cloudfn-method");
const remote_api_1 = require("./remote-api");
const utils_1 = require("./utils");
const config_1 = require("./config");
/**
 * 调用数据源方法
 * @param cfnParams
 * @returns 调用结果
 */
async function invokeDataSource(cfnParams) {
    const dataSource = config_1.getDataSource(cfnParams.dataSource);
    const defaultParams = config_1.getDefaultParams(dataSource.name);
    let fnParams = cfnParams.params || {};
    if (defaultParams) {
        fnParams = utils_1.mergeDeep(defaultParams, fnParams);
    }
    const context = await fn_context_1.getSpecificContext({ dataSource });
    if (dataSource.type === 'database') {
        // 在此不再检查内置方法是否启用, 默认均启用, 由后台做 启用 及 权限的校验
        if (builtin_method_1.isBuiltinMethod(cfnParams.methodName)) {
            return builtin_method_1.callBuiltinMethod(Object.assign(Object.assign({}, cfnParams), { dataSourceName: dataSource.name }), context);
        }
    }
    const methodCfg = getMethodCfg(dataSource, cfnParams.methodName);
    if (!methodCfg) {
        throw new utils_1.TCBError(404, `method ${cfnParams.methodName} not found in datasource ${dataSource.name}`);
    }
    // 是否正在测试当前方法
    const isTestMode = config_1.isMethodInTestMode(dataSource.name, methodCfg.name);
    // 方法被禁用且未在数据源控制台中进行方法测试
    if (methodCfg.disabled && !isTestMode) {
        throw new utils_1.TCBError(403, `method ${cfnParams.methodName} not enabled in datasource ${dataSource.name}`);
    }
    // TODO: 权限功能上线前确认
    // const hasPermission = await hasMethodPermission(dataSource.id, cfnParams.methodName);
    // if (!hasPermission) {
    //   throw new Error(`you don't have permission to invoke method ${cfnParams.methodName} \
    // in datasource ${dataSource.title}(${dataSource.name})`);
    // }
    if (methodCfg.type === 'http') {
        return http_method_1.callHttpMethod(Object.assign(Object.assign({}, cfnParams), { params: fnParams, methodCfg,
            isTestMode }), context);
    }
    if (methodCfg.type === 'cloud-function') {
        return cloudfn_method_1.callCloudFnMethod(Object.assign(Object.assign({}, cfnParams), { params: fnParams, methodCfg }), context);
    }
}
exports.invokeDataSource = invokeDataSource;
/**
 * 获取方法配置描述
 */
function getMethodCfg(dsConfig, methodName) {
    if (dsConfig && Array.isArray(dsConfig.methods)) {
        return dsConfig.methods.find(item => (item === null || item === void 0 ? void 0 : item.name) === methodName);
    }
}
exports.getMethodCfg = getMethodCfg;
/**
 * 检查数据源自定义方法方法是否有权限
 * @param dataSourceId 数据源ID (非数据源name)
 * @param methodName 方法名
 * @returns boolean
 */
async function hasMethodPermission(dataSourceId, methodName) {
    if (!dataSourceId)
        throw new Error('datasource info not found!');
    const resourceId = `${dataSourceId}#${methodName}`;
    try {
        const result = await remote_api_1.fetchWeda({
            action: 'DescribeResourcesPermission',
            data: {
                ResourceType: 'dataSource',
                ResourceIdList: [resourceId],
            },
        }).then(res => res.Data);
        if (result === null || result === void 0 ? void 0 : result.length) {
            const item = result.find((n) => n.ResourceId === resourceId);
            return item === null || item === void 0 ? void 0 : item.IsAccess;
        }
        return false;
    }
    catch (error) {
        throw new Error(`failed to call \`DescribeResourcesPermission\`: ${error.message}`);
    }
}
exports.hasMethodPermission = hasMethodPermission;
